@@ -1,9 +1,9 @@
+import asyncio
 import logging
-import os
+import platform
 import signal
-import threading
-import time
 import warnings
+import threading
 from io import StringIO
 from logging.handlers import TimedRotatingFileHandler
 from typing import Optional, Any, Dict
@@ -13,7 +13,6 @@ import yaml
 from pyobs.object import get_object
 from pyobs.modules import Module
 from pyobs.utils.config import pre_process_yaml
-from pyobs.utils.logger import DuplicateFilter
 
 # just init logger with something here, will be overwritten in __init__
 log = logging.getLogger(__name__)
@@ -22,8 +21,7 @@ log = logging.getLogger(__name__)
 class Application:
     """Class for initializing and shutting down a pyobs process."""
 
-    def __init__(self, config: str, log_file: Optional[str] = None, log_level: str = 'info', log_rotate: bool = False,
-                 **kwargs: Any):
+    def __init__(self, config: str, log_file: Optional[str] = None, log_level: str = "info", **kwargs: Any):
         """Initializes a pyobs application.
 
         Args:
@@ -37,7 +35,7 @@ class Application:
         self._config = config
 
         # formatter for logging, and list of logging handlers
-        formatter = logging.Formatter('%(asctime)s [%(levelname)s] %(filename)s:%(lineno)d %(message)s')
+        formatter = logging.Formatter("%(asctime)s [%(levelname)s] %(filename)s:%(lineno)d %(message)s")
         handlers = []
 
         # create stdout logging handler
@@ -47,13 +45,11 @@ class Application:
 
         # create file logging handler, if log file is given
         if log_file is not None:
-            file_handler: logging.FileHandler
-            if log_rotate:
-                # create automatically rotated log
-                file_handler = TimedRotatingFileHandler(log_file, when='W0')
+            # in Windows, append a FileHandler, otherwise we use a WatchedFileHandler, which works well with logrotate
+            if platform.system() == "Windows":
+                file_handler = logging.FileHandler(log_file)
             else:
-                # create simple file log
-                file_handler = logging.FileHandler(log_file, mode='a')
+                file_handler = logging.handlers.WatchedFileHandler(log_file)
 
             # add log file handler
             file_handler.setFormatter(formatter)
@@ -62,10 +58,10 @@ class Application:
         # basic setup
         logging.basicConfig(handlers=handlers, level=logging.getLevelName(log_level.upper()))
         logging.captureWarnings(True)
-        warnings.simplefilter('always', DeprecationWarning)
+        warnings.simplefilter("always", DeprecationWarning)
 
         # disable tornado logger
-        logging.getLogger('tornado.access').disabled = True
+        logging.getLogger("tornado.access").disabled = True
 
         # set pyobs logger
         global log
@@ -74,79 +70,77 @@ class Application:
         # hack threading to set thread names on OS level
         self._hack_threading()
 
-        # init module with empty one
-        self._module: Module = Module()
+        # load config
+        log.info("Loading configuration from {0:s}...".format(self._config))
+        with StringIO(pre_process_yaml(self._config)) as f:
+            cfg: Dict[str, Any] = yaml.safe_load(f)
+
+        # create module and open it
+        log.info("Creating module...")
+        self._module = get_object(cfg, Module)
 
     def run(self) -> None:
+        """Run app."""
+
+        # signals
+        loop = asyncio.get_event_loop()
+        for sig in (signal.SIGTERM, signal.SIGINT):
+            loop.add_signal_handler(sig, self._signal_handler, sig)
+
+        # run main task forever
+        loop.create_task(self._main())
+        loop.run_forever()
+
+        # main finished, cancel all tasks
+        tasks = asyncio.all_tasks(loop=loop)
+        for t in tasks:
+            t.cancel()
+        group = asyncio.gather(*tasks, return_exceptions=True)
+        loop.run_until_complete(group)
+
+        # finished
+        loop.close()
+
+    def _signal_handler(self, sig) -> None:
+        """React to signals and quit module."""
+
+        # stop loop
+        loop = asyncio.get_running_loop()
+        loop.stop()
+
+        # reset signal handlers
+        log.info(f"Got signal: {sig!s}, shutting down.")
+        loop.remove_signal_handler(signal.SIGTERM)
+        loop.add_signal_handler(signal.SIGINT, lambda: None)
+
+    async def _main(self) -> None:
         """Actually run the application."""
 
         # everything in a try/except/finally, so that we can shut down gracefully
         try:
-            # load config
-            log.info('Loading configuration from {0:s}...'.format(self._config))
-            with StringIO(pre_process_yaml(self._config)) as f:
-                cfg: Dict[str, Any] = yaml.safe_load(f)
-
-            # create module and open it
-            log.info('Creating module...')
-            self._module = get_object(cfg, Module)
-            log.info('Opening module...')
-            self._module.open()
-            log.info('Started successfully.')
+            # open module
+            log.info("Opening module...")
+            await self._module.open()
+            log.info("Started successfully.")
 
             # run module
-            self._run()
+            await self._module.main()
 
         except:
             # some exception was thrown
-            log.exception('Something went wrong.')
+            log.exception("Something went wrong.")
 
         finally:
             # shutting down
-            log.info('Shutting down...')
+            log.info("Shutting down...")
 
             # close module
             if self._module is not None:
-                log.info('Closing module...')
-                self._module.close()
-
-            # still threads running?
-            if threading.active_count() > 1:
-                # get logger
-                wait_logger = logging.getLogger(__name__ + ':wait')
-                wait_logger.addFilter(DuplicateFilter())
-
-                # wait for them to end
-                start_shutdown = time.time()
-                while threading.active_count() > 1:
-                    # print threads
-                    names = [t.name for t in threading.enumerate() if t != threading.current_thread()]
-                    wait_logger.info('Waiting for threads to close: ' + ','.join(names))
-
-                    # after 30 seconds, kill everything
-                    if time.time() - start_shutdown > 30:
-                        wait_logger.error('Threads did not close gracefully, forcing exit...')
-                        os._exit(0)
-
-                    # wait a little
-                    time.sleep(2)
+                log.info("Closing module...")
+                await self._module.close()
 
             # finished
-            log.info('Finished shutting down.')
-
-    def _run(self) -> None:
-        """Method that finally runs the module, can be overridden by derived classes."""
-
-        # add signal handlers
-        signal.signal(signal.SIGTERM, self._signal_handler)
-        signal.signal(signal.SIGINT, self._signal_handler)
-
-        # run module
-        self._module.main()
-
-    def _signal_handler(self, signum: Any, frame: Any) -> None:
-        """React to signals and quit module."""
-        self._module.quit()
+            log.info("Finished shutting down.")
 
     def _hack_threading(self) -> None:
         """Bad hack to set thread name on OS level."""
@@ -164,8 +158,8 @@ class Application:
             threading.Thread._bootstrap = _thread_name_hack  # type: ignore
 
         except ImportError:
-            log = logging.getLogger('pyobs')
-            log.warning('prctl module is not installed. You will not be able to see thread names')
+            logger = logging.getLogger("pyobs")
+            logger.warning("prctl module is not installed. You will not be able to see thread names")
 
             def set_thread_name(name: str) -> None:
                 pass
@@ -195,4 +189,4 @@ class GuiApplication(Application):
         self._qapp.exec()
 
 
-__all__ = ['Application', 'GuiApplication']
+__all__ = ["Application", "GuiApplication"]
