@@ -8,7 +8,7 @@ procedure calls. The base class for all interfaces in *pyobs* is:
 Modules need to implement the required interfaces. For instance, if a module operates a camera, it probably should
 implement :class:`~pyobs.interfaces.ICamera`.
 """
-__title__ = 'Interfaces'
+__title__ = "Interfaces"
 
 from .IAbortable import IAbortable
 from .IAcquisition import IAcquisition
@@ -50,6 +50,7 @@ from .ITelescope import ITelescope
 from .ITemperatures import ITemperatures
 from .IOffsetsAltAz import IOffsetsAltAz
 from .IOffsetsRaDec import IOffsetsRaDec
+from .ISpectrograph import ISpectrograph
 from .IVideo import IVideo
 from .IWeather import IWeather
 from .IWindow import IWindow
