@@ -1,6 +1,6 @@
+import asyncio
 import os
-from typing import Optional, Any, AnyStr
-
+from typing import Optional, Any, AnyStr, List
 import paramiko
 import paramiko.sftp
 
@@ -87,5 +87,38 @@ class SSHFile(VFSFile):
     async def write(self, s: AnyStr) -> None:
         self._fd.write(s)
 
+    @staticmethod
+    async def listdir(path: str, **kwargs: Any) -> List[str]:
+        """Returns content of given path.
+
+        Args:
+            path: Path to list.
+            kwargs: Parameters for specific file implementation (same as __init__).
+
+        Returns:
+            List of files in path.
+        """
+
+        # connect
+        ssh = paramiko.SSHClient()
+        ssh.load_system_host_keys()
+        ssh.connect(
+            kwargs["hostname"],
+            port=kwargs["port"] if "port" in kwargs else 22,
+            username=kwargs["username"],
+            password=kwargs["password"] if "password" in kwargs else None,
+            key_filename=kwargs["keyfile"] if "keyfile" in kwargs else None,
+        )
+        sftp = ssh.open_sftp()
+
+        # list files in path
+        loop = asyncio.get_running_loop()
+        files = await loop.run_in_executor(None, sftp.listdir, os.path.join(kwargs["root"], path))
+
+        # disconnect and return list
+        sftp.close()
+        ssh.close()
+        return files
+
 
 __all__ = ["SSHFile"]
